package App::Pods2Site::SiteBuilder::None;

use strict;
use warnings;

our $VERSION = '1.003';
my $version = $VERSION;
$VERSION = eval $VERSION;

use base qw(App::Pods2Site::AbstractSiteBuilder);

# do nothing

1;
